#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "vbe.h"

char *snip(char *string)
{
    int i;

    /* This is always a 13 character buffer */
    /* and it's not always terminated. */
    string[12] = '\0';

    while (((i = strlen(string)) > 0) && (isspace(string[i - 1]) ||
					  (string[i - 1] == '\n')
					  || (string[i - 1] == '\r')))
	string[i - 1] = '\0';
    return string;
}

int main(int argc, char **argv)
{
    int i, j;
    unsigned char hmin, hmax, vmin, vmax;
    struct vbe_info *vbe_info = vbe_get_vbe_info();
    struct vbe_modeline *modelines = vbe_get_edid_modelines();
    struct vbe_edid1_info *edid_info = vbe_get_edid_info();
    struct vbe_edid_monitor_descriptor *monitor = NULL;
    struct vbe_edid_detailed_timing *timing = NULL;

    /* eloszor kitalaljuk a hsync es vsync-et */
    vbe_get_edid_ranges(&hmin, &hmax, &vmin, &vmax);
    if (!hmin && !hmax && !vmin && !vmax) {
	hmin = 28;
	hmax = 33;
	vmin = 43;
	vmax = 72;
	for (j = 0; modelines && (modelines[j].refresh != 0); j++) {
	    if (hmin == 0)
		hmin = modelines[j].hfreq;
	    if (modelines[j].hfreq)
		hmin =
		    (hmin <
		     modelines[j].hfreq) ? hmin : modelines[j].hfreq;
	    if (modelines[j].hfreq)
		hmax =
		    (hmax >
		     modelines[j].hfreq) ? hmax : modelines[j].hfreq;
	    if (vmin == 0)
		vmin = modelines[j].vfreq;
	    if (modelines[j].vfreq)
		vmin =
		    (vmin <
		     modelines[j].vfreq) ? vmin : modelines[j].vfreq;
	    if (modelines[j].vfreq)
		vmax =
		    (vmax >
		     modelines[j].vfreq) ? vmax : modelines[j].vfreq;
	}
    }

    /* 1. sor: monitor neve */
    if (edid_info)
	for (i = 0; i < 4; i++) {
	    monitor = &edid_info->monitor_details.monitor_descriptor[i];
	    timing = &edid_info->monitor_details.detailed_timing[i];
	    if (monitor->type == vbe_edid_monitor_descriptor_name) {
		printf("%s", snip(monitor->data.string));
	    }
    } else
	printf("unknown");
    printf("\n");

    /* 2. sor: monitor sorozatszama */
    if (edid_info)
	for (i = 0; i < 4; i++) {
	    monitor = &edid_info->monitor_details.monitor_descriptor[i];
	    timing = &edid_info->monitor_details.detailed_timing[i];
	    if (monitor->type == vbe_edid_monitor_descriptor_serial) {
		printf("%s", snip(monitor->data.string));
	    }
	}
    printf("\n");

    /* 3. sor: monitor egyeb szovege */
    if (edid_info)
	for (i = 0; i < 4; i++) {
	    monitor = &edid_info->monitor_details.monitor_descriptor[i];
	    timing = &edid_info->monitor_details.detailed_timing[i];
	    if (monitor->type == vbe_edid_monitor_descriptor_ascii) {
		printf("%s", snip(monitor->data.string));
	    }
	}
    printf("\n");

    /* 4-7. sor: HsyncMin, HsyncMan, VsyncMin, VsyncMax */
    printf("%d\n%d\n%d\n%d\n", hmin, hmax, vmin, vmax);

    /* 8-9: horiz, vert size (cm) */
    if (edid_info)
	printf("%d\n%d\n",
	       edid_info->max_size_horizontal,
	       edid_info->max_size_vertical);
    else
	printf("0\n0\n");

    /* 10. sor: videomem */
    if (vbe_info)
	if (strcasestr(vbe_info->oem_name.string, "intel")
	    && strstr(vbe_info->oem_name.string, "810"))
	    printf("4096\n");
	else
	    printf("%d\n", vbe_info->memory_size * 64);
    else
	printf("512\n");

    /* 11: OEM Name */
    printf("%s\n", vbe_info ? vbe_info->oem_name.string : "");

    /* 12-14: Vendor name, Product name, Product revision */
    if (vbe_info && vbe_info->version[1] >= 3)
	printf("%s\n%s\n%s\n", vbe_info->vendor_name.string,
	       vbe_info->product_name.string,
	       vbe_info->product_revision.string);
    else
	printf("\n\n\n");

    /* vegul a modelineok (width, height, refresh(interlaced), modeline */
    for (j = 0; modelines && (modelines[j].refresh != 0); ++j) {
	if (modelines[j].modeline)
	    printf("%d %d %1.1f%s %s\n",
		   modelines[j].width, modelines[j].height,
		   modelines[j].refresh,
		   modelines[j].interlaced ? "i" : "",
		   modelines[j].modeline);
    }
    return 0;
}
